/*
   Naive Bayes algorithm implementation for the
   Titanic test case
   (c) Lesept, September 2021
*/
#include <Arduino.h>
#include "FS.h"
#include "SPIFFS.h"
#include "csv_Vector.h"
csv_Vector myVector;
#include "NaiveBayes.h"

#define FORMAT_SPIFFS_IF_FAILED true
const char * path = "/Clean_Dataset.csv";

#define nData 707
#define nFeatures 4
#define nClasses 2
NB Titanic(nData, nFeatures, nClasses);
std::vector<Data>dataset;

int inputInt() {
  String value;
  while (Serial.available() == 0) {}
  value = Serial.readString();

  if (value.length() > 0) {
    int n = value.toInt();
    Serial.println(n);
    return n;
  }
  return 0;
}

char inputChar() {
  String value;
  while (Serial.available() == 0) {}
  value = Serial.readString();

  if (value.length() > 0) {
    value.toUpperCase();
    char n = value.charAt(0);
    Serial.println(n);
    return n;
  }
  return 0;
}


void setup() {
  char buffer[50];
  Serial.begin(115200);
  myVector.init(";");

  // Mount SPIFFS
  if (!SPIFFS.begin(FORMAT_SPIFFS_IF_FAILED)) {
    Serial.println("SPIFFS Mount Failed");
    return;
  }

  // Open dataset file
  File file = SPIFFS.open(path);
  if (!file || file.isDirectory()) {
    Serial.println("- failed to open file for reading");
    return;
  }

  // Read file
  Serial.printf("Reading data from file: %s\n", path);
  int n = 0;

  while (file.available()) {
    if (!myVector.readLine (file, buffer)) Serial.println("Read error"); // Read a line
    if (n != 0) { // Extract numbers from the line
      std::vector<uint8_t> x = myVector.readVector (buffer, nFeatures, 2);
      Titanic.addDataCat(x, dataset);
    } else Serial.println(buffer); // or print the line if it is the first
    n++;
  }
  file.close();
  Serial.printf("Read %d data from file\n", n - 1);

  // Fit the dataset
  Titanic.fit(dataset);

  /* To make a prediction on a single set of data, create an array containing the 
    following values: {classe, sex [1 for male, 0 for female], age, fare / 2}
    int data[] = {2, 0, 25, 27}; // --> second class, female, 25 yo, 54£
    std::vector<uint8_t> x (data, data + 4);
    uint8_t predict = Titanic.predictCatFit(x, dataset);
    Serial.printf("Prediction %d\n", predict);
  */
}

void loop() {
  bool ok;
  Serial.println ("\n         _");
  Serial.println ("        / \\");
  Serial.println ("    __/ _  \\_                        ,   ,   ,");
  Serial.println ("   /_  -  \\  \\                      ,: ,: ,:  ");
  Serial.println ("  / / /     \\ \\                  __||_||_||_||___");
  Serial.println (" |    |  _  / |             ____[\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"]___");
  Serial.println (" /   /     \\   \\            \\ @ ''''''''''''''''''''/");
  Serial.println (" ~~^~^~~~^~^~^~~^~^~^~~~^~^~^^~^~^~^~^~^~^~^~~^~^~^~^~~^~^");
  Serial.println ("Did you survive the TITANIC ? ");
  ok = false;
  int age;
  while (!ok) {
    Serial.println("Enter your age (1 to 80)");
    age = inputInt();
    if (age > 0 && age < 81) ok = true;
  }

  ok = false;
  int sex;
  while (!ok) {
    Serial.println("Are you Male or Female (M or F) ? ");
    char c = inputChar();
    if (c == 'M' || c == 'F') ok = true;
    sex = (c == 'M') ? 1 : 0;
  }

  ok = false;
  int classe;
  while (!ok) {
    Serial.println("In which class did you travel (1, 2 or 3) ? ");
    classe = inputInt();
    if (classe > 0 && classe < 4) ok = true;
  }

  ok = false;
  int fare;
  while (!ok) {
    Serial.println("What is the fare of your trip (2 to 510 £) ? ");
    fare = inputInt();
    if (fare > 1 && fare < 511) ok = true;
  }

  int data[] = {classe, sex, age, fare / 2};
  std::vector<byte> x (data, data + 4);
  uint8_t prediction = Titanic.predictCat(x, dataset);
  for (int i = 0; i < 5 + random(15); i++) {
    Serial.print(".");
    delay(random(600));
  }
  Serial.printf((prediction) ? "\nCongratulations, you survived !!!\n" : "\nSorry, you died...\n");
}
